﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinSchedule;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinScrollBar;
using System.Diagnostics;
using Infragistics.Shared;
using Infragistics.Win.UltraWinSchedule.MonthViewMulti;
using Microsoft.VisualBasic.Devices;
using Infragistics.Win;

namespace UltraMonthViewMultiDateTimeEditor
{
    public partial class CustomMonthViewMulti : UltraMonthViewMulti
    {

        #region Properties
        public UltraComboEditor comboEditor;
        public UltraDateTimeEditor dateTimeEditor;
        private DateTime firstMonth;
        private ScrollArrowUIElement BackScrollElement;
        private ScrollArrowUIElement ForwardScrollElement;

        private DropDownStyle dropDownStyle = DropDownStyle.None;
        [Category("Layout")]
        [Description("Define the dropdown style")]
        public DropDownStyle DropDownStyle
        {
            get
            {
                return dropDownStyle;
            }
            set
            {
                dropDownStyle = value;
                switch (dropDownStyle)
                {
                    case UltraMonthViewMultiDateTimeEditor.DropDownStyle.None:
                        this.comboEditor.Visible = false;
                        this.dateTimeEditor.Visible = false;
                        this.CreationFilter = null;

                        break;
                    case UltraMonthViewMultiDateTimeEditor.DropDownStyle.UseComboStyle:
                        this.comboEditor.Location = new Point(this.Size.Width - 21, 0);
                        this.comboEditor.Visible = true;
                        this.dateTimeEditor.Visible = false;
                        this.CreationFilter = new CreatingDropDownUIElement();
                        break;

                    case UltraMonthViewMultiDateTimeEditor.DropDownStyle.UseMonthStyle:
                        this.dateTimeEditor.Location = new Point(this.Size.Width - 21, 0);
                        this.comboEditor.Visible = false;
                        this.dateTimeEditor.Visible = true;
                        this.CreationFilter = new CreatingDropDownUIElement();
                        break;
                }
            }
        }
        #endregion Properties

        #region Constructors
        public CustomMonthViewMulti()
        {
            InitializeComponent();
            this.MonthScrollButtonsVisible = Infragistics.Win.DefaultableBoolean.False;
            this.BeforeDisplayMonthPopup += new BeforeDisplayMonthPopupEventHandler(CustomMonthViewMulti_BeforeDisplayMonthPopup);

            this.InitComboEditor();
            this.InitDateTimeEditor();
            this.DropDownStyle = UltraMonthViewMultiDateTimeEditor.DropDownStyle.None;
        }

        public void InitComboEditor()
        {
            this.comboEditor = new UltraComboEditor();
            this.comboEditor.Parent = this;
            this.comboEditor.Size = new Size(1, 1);
            this.comboEditor.Location = new Point(this.Size.Width - 21, 0);
            this.comboEditor.DropDownListWidth = 130;
            this.comboEditor.MaxDropDownItems = 6;
            this.comboEditor.BeforeDropDown += new CancelEventHandler(comboEditor_BeforeDropDown);
            this.comboEditor.MouseEnterElement += new Infragistics.Win.UIElementEventHandler(comboEditor_MouseEnterElement);
            this.comboEditor.DropDownListAlignment = Infragistics.Win.DropDownListAlignment.Right;
            this.comboEditor.SelectionChanged += new EventHandler(comboEditor_SelectionChanged);
        }

        private void InitDateTimeEditor()
        {
            this.dateTimeEditor = new UltraDateTimeEditor();
            this.dateTimeEditor.Parent = this;
            this.dateTimeEditor.Size = new Size(1, 1);
            this.dateTimeEditor.Location = new Point(this.Size.Width - 21, 0);
            this.dateTimeEditor.ValueChanged += new EventHandler(dateTimeEditor_ValueChanged);
        }

        #endregion Constructors

        #region ProtectedOverrides

        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);
        }

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);
            if (this.comboEditor != null) this.comboEditor.Location = new Point(this.Size.Width - 21, 0);
            if (this.dateTimeEditor != null) this.dateTimeEditor.Location = new Point(this.Size.Width - 21, 0);
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            base.OnPaint(pe);
        }

        void CustomMonthViewMulti_BeforeDisplayMonthPopup(object sender, BeforeDisplayMonthPopupEventArgs e)
        {
            e.Cancel = true;
        }

        #endregion ProtectedOverrides

        #region DateTimeEditor

        void dateTimeEditor_ValueChanged(object sender, EventArgs e)
        {
            this.InternalEnsureVisible(this.dateTimeEditor.DateTime);
        }

        #endregion DateTimeEditor

        #region ComboEditor

        void comboEditor_SelectionChanged(object sender, EventArgs e)
        {
            this.InternalEnsureVisible((DateTime)this.comboEditor.SelectedItem.DataValue);
        }

        void ForwardScrollElement_ElementClick(object sender, Infragistics.Win.UIElementEventArgs e)
        {
            DateTime dt = (DateTime)this.comboEditor.Items[this.comboEditor.Items.Count - 1].DataValue;
            this.comboEditor.Items.Add(dt.AddMonths(1), String.Format("{0:y}", dt.AddMonths(1)));
        }

        void BackScrollElement_ElementClick(object sender, Infragistics.Win.UIElementEventArgs e)
        {
            DateTime dt = (DateTime)this.comboEditor.Items[0].DataValue;
            this.comboEditor.Items.Insert(0, dt.AddMonths(-1), String.Format("{0:y}", dt.AddMonths(-1)));
        }

        void comboEditor_MouseEnterElement(object sender, Infragistics.Win.UIElementEventArgs e)
        {
            if (e.Element.GetType() == typeof(ScrollArrowUIElement))
            {
                BackScrollElement = e.Element.Parent.ChildElements[0] as ScrollArrowUIElement;
                ForwardScrollElement = e.Element.Parent.ChildElements[1] as ScrollArrowUIElement;

                if (BackScrollElement != null) BackScrollElement.ElementClick += new Infragistics.Win.UIElementEventHandler(BackScrollElement_ElementClick);
                if (ForwardScrollElement != null) ForwardScrollElement.ElementClick += new Infragistics.Win.UIElementEventHandler(ForwardScrollElement_ElementClick);
            }
        }

        void comboEditor_BeforeDropDown(object sender, CancelEventArgs e)
        {
            firstMonth = new DateTime(this.FirstMonth.Year.YearNumber, this.FirstMonth.MonthNumber, 1);
            for (int k = -12; k < 12; k++)
                this.comboEditor.Items.Add(firstMonth.AddMonths(k), String.Format("{0:y}", firstMonth.AddMonths(k)));
        }

        #endregion ComboEditor

    }

    public enum DropDownStyle
    {
        None = 0,
        UseComboStyle = 1,
        UseMonthStyle = 2,
    }

    public class CreatingDropDownUIElement : IUIElementCreationFilter
    {
        public void AfterCreateChildElements(UIElement parent)
        {
            MonthUIElement El = parent as MonthUIElement;
            int state = 0;

            if (El != null)
            {
                ScrollButtonUIElement button = new ScrollButtonUIElement(El, ScrollButton.Down);
                button.ElementClick += new UIElementEventHandler(button_ElementClick);
                El.ChildElements.Add(button);


                if (((CustomMonthViewMulti)El.Control).MonthScrollButtonsVisible == DefaultableBoolean.True) state++;
                if (((CustomMonthViewMulti)El.Control).YearScrollButtonsVisible == DefaultableBoolean.True) state++;

                
                if ((El.Rect.X + El.Rect.Width) > El.GetAncestor(typeof(MonthAreaUIElement)).Rect.Width && El.Rect.Y < 10)
                {
                    switch(state)
                    {
                        case 0:
                            button.Rect = new Rectangle(El.Rect.X + El.Rect.Width - 18, El.Rect.Y + 2, 15, 15);
                            break;

                        case 1:
                            button.Rect = new Rectangle(El.Rect.X + El.Rect.Width - 35, El.Rect.Y + 2, 15, 15);
                            break;

                        case 2:
                            button.Rect = new Rectangle(El.Rect.X + El.Rect.Width - 48, El.Rect.Y + 2, 15, 15);
                            break;
                    }
                }
                else
                {
                    button.Rect = new Rectangle(El.Rect.X + El.Rect.Width - 18, El.Rect.Y + 2, 15, 15);
                }
            }
        }

        void button_ElementClick(object sender, UIElementEventArgs e)
        {
            ((CustomMonthViewMulti)e.Element.Control).comboEditor.Location = new Point(e.Element.Parent.Rect.X + e.Element.Parent.Rect.Width, e.Element.Parent.Rect.Y);
            ((CustomMonthViewMulti)e.Element.Control).comboEditor.DropDown();

            ((CustomMonthViewMulti)e.Element.Control).dateTimeEditor.Location = new Point(e.Element.Parent.Rect.X + e.Element.Parent.Rect.Width, e.Element.Parent.Rect.Y);
            ((CustomMonthViewMulti)e.Element.Control).dateTimeEditor.DropDown();
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

    }
}
